package com.kexun.common.utils;

import org.apache.http.*;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.springframework.util.StringUtils;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.nio.charset.Charset;
import java.util.*;

public class HttpUtils {


//    private static final String ENCODING = "utf-8";

    public static String doPostForm(String url, Map<String, String> params) {
        return doPostFormDetail(url, params, null);
    }

    public static String doPostFormDetail(String url, Map<String, String> params, Map<String, String> header) {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        String rest = "";
        CloseableHttpResponse response = null;
        try {
            HttpPost httpPost = new HttpPost(url);
            if (header != null) {
                Set<String> keySet = header.keySet();
                for (String key : keySet) {
                    String value = header.get(key);
                    httpPost.addHeader(key, value);
                }
            }
            if (params != null) {
                List<NameValuePair> nameValuePairs = new ArrayList<>();
                Set<String> keys = params.keySet();
                for (String key : keys) {
                    nameValuePairs.add(new BasicNameValuePair(key, params.get(key)));
                }
                UrlEncodedFormEntity entity = new UrlEncodedFormEntity(nameValuePairs, "UTF-8");
                httpPost.setEntity(entity);
            }
            response = httpClient.execute(httpPost);
            if (response.getStatusLine().getStatusCode() == 200) {
                rest = EntityUtils.toString(response.getEntity(), "UTF-8");
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                httpClient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return rest;
    }

    public static String doGet(String url, Map<String, String> param, String encoding) {
        // 创建Httpclient对象
        CloseableHttpClient httpclient = HttpClients.createDefault();
        String resultString = "";
        CloseableHttpResponse response = null;
        try {
            // 创建uri
            URIBuilder builder = new URIBuilder(url);
            if (param != null) {
                for (String key : param.keySet()) {
                    builder.addParameter(key, param.get(key));
                }
            }
            URI uri = builder.build();

            // 创建http GET请求
            HttpGet httpGet = new HttpGet(uri);

            // 执行请求
            response = httpclient.execute(httpGet);
            // 判断返回状态是否为200
            if (response.getStatusLine().getStatusCode() == 200) {
                resultString = EntityUtils.toString(response.getEntity(), encoding == null ? "utf-8" : encoding);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                httpclient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultString;
    }

    public static String doGet(String url) {
        return doGet(url, null, null);
    }

    public static String doGet(String url, String encoding) {
        return doGet(url, null, encoding);
    }

    public static String doPost(String url, Map<String, String> param, Map<String, String> header, String encoding) {
        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            // 创建参数列表
            if (param != null) {
                List<NameValuePair> paramList = new ArrayList<>();
                for (String key : param.keySet()) {
                    paramList.add(new BasicNameValuePair(key, param.get(key)));
                }
                // 模拟表单
                UrlEncodedFormEntity entity = new UrlEncodedFormEntity(paramList, encoding);
                httpPost.setEntity(entity);
                if (header != null) {
                    Set<Map.Entry<String, String>> entries = header.entrySet();
                    for (Map.Entry<String, String> entry : entries) {
                        httpPost.addHeader(entry.getKey(), entry.getValue());
                    }

                }

            }
            // 执行http请求
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), encoding);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultString;
    }

    public static String doPost(String url) {
        return doPost(url, null, null, "utf-8");
    }

    public static String doPost(String url, Map<String, String> param) {
        return doPost(url, param, null, "utf-8");
    }


    public static String doPostJson(String url, String json, Map<String, String> headerMap) {
        return doPostJson(url, headerMap, null, null, json, "utf-8");
    }

    public static String doPostJson(String url, String json) {
        return doPostJson(url, null, null, null, json, "utf-8");
    }

    public static String doPostJsonProxy(String url, String json, Map<String, String> headerMap, String ip, int port) {
        return doPostJson(url, headerMap, ip, port, json, "utf-8");
    }

    public static String doPostJsonProxy(String url, String json, String ip, int port) {
        return doPostJson(url, null, ip, port, json, "utf-8");
    }

    /**
     * 请求的参数类型为json
     *
     * @param url
     * @param json
     * @return {username:"",pass:""}
     */
    public static String doPostJson(String url, Map<String, String> headerMap, String ip, Integer port, String json, String encoding) {
        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpHost proxy = null;
        if (!StringUtils.isEmpty(ip) && port != null) {
            proxy = new HttpHost(ip, port);
        }
        RequestConfig config = RequestConfig.custom().setConnectTimeout(60000).setProxy(proxy).build();
        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            httpPost.setConfig(config);

            if (headerMap != null) {
                Iterator headerIterator = headerMap.entrySet().iterator();          //循环增加header
                while (headerIterator.hasNext()) {
                    Map.Entry<String, String> elem = (Map.Entry<String, String>) headerIterator.next();
                    httpPost.addHeader(elem.getKey(), elem.getValue());
                }
            }

            // 创建请求内容
            StringEntity entity = new StringEntity(json, ContentType.APPLICATION_JSON);
            httpPost.setEntity(entity);
            // 执行http请求
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), encoding);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultString;
    }

    /**
     * 请求的参数类型为xml,支持SOAP1.2
     *
     * @param url
     * @param xml
     * @return {username:"",pass:""}
     */
    public static String doPostXml(String url, String xml, String soapname, String encoding) {
        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            // 设置SOAPAction，支持soap1.2
            httpPost.setHeader("SOAPAction", soapname);
            // 创建请求内容
            StringEntity entity = new StringEntity(xml, ContentType.TEXT_XML);
            httpPost.setEntity(entity);
            // 执行http请求
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), encoding);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultString;
    }

    /**
     * 请求的参数类型为xml
     *
     * @param url
     * @param xml
     * @return {username:"",pass:""}
     */
    public static String doPostXml(String url, String xml, String encoding) {
        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            // 创建请求内容
            StringEntity entity = new StringEntity(xml, ContentType.create("text/xml", Consts.UTF_8));
            httpPost.setEntity(entity);
            // 执行http请求
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), encoding);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultString;
    }

    public static String doPostWebservice(String url, String xml, String encoding) throws Exception {
        // 创建Httpclient对象
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        String resultString = "";
        try {
            // 创建Http Post请求
            HttpPost httpPost = new HttpPost(url);
            httpPost.setHeader("Content-Type", "text/xml;charset=UTF-8");
            StringEntity data = new StringEntity(xml, Charset.forName("UTF-8"));
            httpPost.setEntity(data);
            response = httpClient.execute(httpPost);
            resultString = EntityUtils.toString(response.getEntity(), encoding);
        } catch (Exception e) {
            throw e;
        } finally {
            try {
                response.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultString;
    }

    public static String doPostFile(String url, File file)  {
        return doPostFile(url, file, null);
    }

    public static String doPostFile(String url, File file, HashMap<String, String> param) {
        String res = null;
        try {
            CloseableHttpClient httpClient = HttpClients.createDefault();
            HttpPost httppost = new HttpPost(url);
            MultipartEntityBuilder builder = MultipartEntityBuilder.create();
            builder.addBinaryBody("file", file);

            if (param != null) {
                ContentType contentType = ContentType.create(HTTP.PLAIN_TEXT_TYPE, HTTP.UTF_8);
                Set<Map.Entry<String, String>> entrySet = param.entrySet();
                for (Map.Entry<String, String> stringObjectEntry : entrySet) {
                    String key = stringObjectEntry.getKey();
                    String value = stringObjectEntry.getValue();
                    builder.addTextBody(key, value, contentType);
                }
            }
            builder.setMode(HttpMultipartMode.RFC6532);
//        builder.setCharset(Charset.forName("UTF-8"));
            //请求体
            HttpEntity reqEntity = builder.build();
            httppost.setEntity(reqEntity);
//        httppost.setHeader("Content-Type", "multipart/form-data");
            CloseableHttpResponse response = httpClient.execute(httppost);
            HttpEntity entity = response.getEntity();
            if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                res = EntityUtils.toString(entity, "UTF-8");
                response.close();
            } else {
                res = EntityUtils.toString(entity, "UTF-8");
                response.close();
                throw new IllegalArgumentException(res);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return res;
    }


}
