package com.kexun.config;

import com.kexun.common.redis.RedisUtil;
import com.kexun.interceptor.AdminLoginInterceptor;
import com.kexun.interceptor.CorsInterceptor;
import com.kexun.resolver.PlatUserHandlerMethodArgumentResolver;
import com.kexun.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.util.List;

@Component
public class WebMvcConfig implements WebMvcConfigurer {

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private RedisUtil redisUtil;



    //注册登录用户转换器 实现自动注入
    @Override
    public void addArgumentResolvers(List<HandlerMethodArgumentResolver> resolvers) {
        // 策略模式  模板方法
        resolvers.add(new PlatUserHandlerMethodArgumentResolver());
    }

    //拦截器要注册
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        //跨域
        registry.addInterceptor(new CorsInterceptor()).addPathPatterns("/**").order(-1);
        registry.addInterceptor(new AdminLoginInterceptor(sysUserService, redisUtil)).addPathPatterns("/admin/**");
    }


}
