package com.kexun.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.kexun.common.JsonFile;
import com.kexun.common.utils.Result;
import com.kexun.entity.*;
import com.kexun.service.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/*
 * 目标：
 * 目前：
 */

@RestController
@RequestMapping("admin/role")
public class RoleController {

    @Autowired
    private SysRoleRightsService sysRoleRightsService;

    @Autowired
    private SysRoleService sysRoleService;
    @Autowired
    private SysRightsService sysRightsService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private UserInfoService userInfoService;
    @Autowired
    private UserModelService userModelService;
    @Autowired
    private UserApprovalService userApprovalService;
    @Autowired
    private UserFieldService userFieldService;

    private JsonFile jsonFile=new JsonFile();
    @GetMapping("list")
    public Result list(String name) {
        QueryWrapper<SysRole> wrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(name)) {
            wrapper.like("role_name", name);
        }
        List<SysRole> list = sysRoleService.list(wrapper);
        return Result.success("ok", list);
    }

    @GetMapping("rights")
    public Result rights() {
        List<SysRights> sysRightsList = sysRightsService.list(new QueryWrapper<SysRights>().eq("parent_id", 0));
        JSONArray rightsList = new JSONArray();
        for (SysRights sysRights : sysRightsList) {
            JSONObject item = new JSONObject();
            item.put("id", sysRights.getId());
            item.put("label", sysRights.getRightName());
            JSONArray children = new JSONArray();
            List<SysRights> subRights = sysRightsService.list(new QueryWrapper<SysRights>().eq("parent_id", sysRights.getId()));
            for (SysRights subRight : subRights) {
                JSONObject subItem = new JSONObject();
                subItem.put("id", subRight.getId());
                subItem.put("label", subRight.getRightName());
                children.add(subItem);
            }
            item.put("children", children);
            rightsList.add(item);
        }
        return Result.success("ok", rightsList);
    }

    @PostMapping("add")
    @Transactional
    public Result add(@RequestBody JSONObject param) {
        String ruleName = param.getString("ruleName");
        JSONArray rights = param.getJSONArray("rights");
        SysRole sysRole = new SysRole();
        sysRole.setRoleName(ruleName);
        sysRole.setCreateTime(new Date());
        sysRoleService.save(sysRole);
        for (int i = 0; i < rights.size(); i++) {
            Long rightId = rights.getLong(i);
            SysRoleRights sysRoleRights = new SysRoleRights();
            sysRoleRights.setRightId(rightId);
            sysRoleRights.setRoleId(sysRole.getId());
            sysRoleRightsService.save(sysRoleRights);
        }
        return Result.success();
    }

    @GetMapping("info")
    public Result info(Long roleId) {

        JSONObject result = new JSONObject();

        SysRole sysRole = sysRoleService.getById(roleId);

        ArrayList<Long> hasIds = new ArrayList<>();
        //查询拥有的权限
        List<SysRoleRights> sysRoleRights = sysRoleRightsService.list(new QueryWrapper<SysRoleRights>().eq("role_id", roleId));
        for (SysRoleRights sysRoleRight : sysRoleRights) {
            SysRights sysRights = sysRightsService.getById(sysRoleRight.getRightId());
            int subCount = sysRightsService.count(new QueryWrapper<SysRights>().eq("parent_id", sysRoleRight.getRightId()));
            if (sysRights.getParentId() == 0 && subCount <= 0) {
                hasIds.add(sysRoleRight.getRightId());
            }

            if (sysRights.getParentId() != 0) {
                hasIds.add(sysRoleRight.getRightId());
            }

        }

        //查询一级权限
        List<SysRights> sysRightsList = sysRightsService.list(new QueryWrapper<SysRights>().eq("parent_id", 0));
        JSONArray rightsList = new JSONArray();
        for (SysRights sysRights : sysRightsList) {
            JSONObject item = new JSONObject();
            item.put("id", sysRights.getId());
            item.put("label", sysRights.getRightName());
            JSONArray children = new JSONArray();
            List<SysRights> subRights = sysRightsService.list(new QueryWrapper<SysRights>().eq("parent_id", sysRights.getId()));
            for (SysRights subRight : subRights) {
                JSONObject subItem = new JSONObject();
                subItem.put("id", subRight.getId());
                subItem.put("label", subRight.getRightName());
                children.add(subItem);
            }
            item.put("children", children);
            rightsList.add(item);
        }
        result.put("roleName", sysRole.getRoleName());
        result.put("rightsList", rightsList);
        result.put("hasIds", hasIds);
        result.put("roleId", roleId);
        return Result.success("ok", result);
    }


    @PostMapping("edit")
    public Result edit(@RequestBody JSONObject param) {
        Long roleId = param.getLong("roleId");
        String ruleName = param.getString("ruleName");
        JSONArray rights = param.getJSONArray("rights");
        HashSet<Long> newIdSet = new HashSet<>();
        HashSet<Long> oldIdSet = new HashSet<>();
        for (int i = 0; i < rights.size(); i++) {
            newIdSet.add(rights.getLong(i));
        }
        List<SysRoleRights> sysRoleRights = sysRoleRightsService.list(new QueryWrapper<SysRoleRights>().eq("role_id", roleId));

        for (SysRoleRights sysRoleRight : sysRoleRights) {
            oldIdSet.add(sysRoleRight.getRightId());
        }
        HashSet<Long> addSet = new HashSet<>();
        HashSet<Long> delSet = new HashSet<>();
        for (Long oldId : oldIdSet) {
            //如果新的的里面不存在之前有的就要删除
            if (!newIdSet.contains(oldId)) {
                delSet.add(oldId);
            }
        }
        for (Long newId : newIdSet) {
            //如果老的的里面存在新的就要添加
            if (!oldIdSet.contains(newId)) {
                addSet.add(newId);
            }
        }
        for (Long rightId : addSet) {
            sysRoleRightsService.addRoleIdAndRightId(roleId, rightId);
        }
        for (Long rightId : delSet) {
            sysRoleRightsService.delRoleIdAndRightId(roleId, rightId);
        }

        SysRole sysRole = sysRoleService.getById(roleId);
        sysRole.setRoleName(ruleName);
        sysRoleService.updateById(sysRole);

        return Result.success();
    }

    @GetMapping("del")
    public Result del(Long roleId) {
        int count = sysUserService.count(new QueryWrapper<SysUser>().eq("role_id", roleId));
        if (count > 0) {
            return Result.error("当前角色关联了" + count + "个用户,请先取消关联");
        }
        sysRoleService.removeById(roleId);
        return Result.success();
    }


}
