package com.kexun.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.kexun.common.utils.MD5Utils;
import com.kexun.common.utils.PageResult;
import com.kexun.common.utils.Result;
import com.kexun.entity.SysRole;
import com.kexun.entity.SysUser;
import com.kexun.service.SysRoleService;
import com.kexun.service.SysUserService;
//import com.sun.org.apache.regexp.internal.RE;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.UUID;

/*
 * 目标：list、info、add、edit、delete、
 * 目前：已实现
 */

@RestController
@RequestMapping("admin/user")
public class UserController {

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private SysRoleService sysRoleService;

    @GetMapping("list")
    public PageResult list(String userName, String phone, Integer page, Integer limit) {
        QueryWrapper<SysUser> w = new QueryWrapper<>();
        if (!StringUtils.isEmpty(userName)) {
            w.like("user_name", userName);
        }
        if (!StringUtils.isEmpty(phone)) {
            w.like("phone", phone);
        }
        IPage<SysUser> iPage = sysUserService.page(new Page<>(page, limit));
        List<SysUser> records = iPage.getRecords();
        JSONArray result = new JSONArray();
        for (SysUser record : records) {
            SysRole sysRole = sysRoleService.getById(record.getRoleId());
            JSONObject item = new JSONObject();
            item.put("userName", record.getUserName());
            item.put("showName", record.getShowName());
            item.put("phone", record.getPhone());
            item.put("role", sysRole.getRoleName());
            item.put("createTime", record.getCreateTime());
            item.put("userId", record.getUserId());
            result.add(item);
        }
        long total = iPage.getTotal();
        return PageResult.success(result, total);
    }

    @PostMapping("add")
    public Result add(@RequestBody JSONObject param) {
        String userName = param.getString("userName");
        SysUser sysUser1 = sysUserService.findByUserName(userName);
        if (sysUser1 != null) {
            return Result.error("用户名已存在");
        }
        String password = param.getString("password");
        Long roleId = param.getLong("roleId");
        String showName = param.getString("showName");
        String phone = param.getString("phone");
        String salt = UUID.randomUUID().toString().replace("-", "");
        String s = MD5Utils.md5slat(password, salt, 2);
        SysUser sysUser = new SysUser();
        sysUser.setUserName(userName);
        sysUser.setPassword(s);
        sysUser.setSalt(salt);
        sysUser.setRoleId(roleId);
        sysUser.setShowName(showName);
        sysUser.setPhone(phone);
        sysUser.setCreateTime(new Date());
        sysUserService.save(sysUser);
        return Result.success();
    }

    @GetMapping("info")
    public Result info(String userId) {
        SysUser sysUser = sysUserService.getById(userId);
        JSONObject result = new JSONObject();
        result.put("userName", sysUser.getUserName());
        result.put("roleId", sysUser.getRoleId());
        result.put("showName", sysUser.getShowName());
        result.put("phone", sysUser.getPhone());
        result.put("userId", sysUser.getUserId());
        return Result.success("", result);
    }

    @PostMapping("edit")
    public Result edit(@RequestBody JSONObject param) {
        String userId = param.getString("userId");
        String userName = param.getString("userName");
        String password = param.getString("password");
        Long roleId = param.getLong("roleId");
        String showName = param.getString("showName");

        SysUser sysUser = sysUserService.getById(userId);
        sysUser.setUserName(userName);
        //填了密码才修改
        if (!StringUtils.isEmpty(password)) {
            String salt = UUID.randomUUID().toString().replace("-", "");
            String s = MD5Utils.md5slat(password, salt, 2);
            sysUser.setPassword(s);
            sysUser.setSalt(salt);
        }
        sysUser.setRoleId(roleId);
        sysUser.setShowName(showName);
        sysUserService.updateById(sysUser);
        return Result.success();
    }

    @GetMapping("del")
    public Result del(SysUser sysUser, String userId) {
        if (sysUser.getUserId() == userId) {
            return Result.error("不能删除当前登录用户");
        }
        sysUserService.removeById(userId);
        return Result.success();
    }

}
