package com.kexun.interceptor;

import com.kexun.common.redis.RedisUtil;
import com.kexun.entity.SysUser;
import com.kexun.exception.ApiRRException;
import com.kexun.service.SysUserService;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;
import org.springframework.web.util.UrlPathHelper;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author lidong
 * @email 3275208135@qq.com
 * @date 2019年5月16日 下午17:16:02
 */
public class AdminLoginInterceptor extends HandlerInterceptorAdapter {
    private SysUserService sysUserService;

    private RedisUtil redisUtil;

    public static final String LOGIN_USER_KEY = "PLAT_USER_KEY";
    public static final String LOGIN_TOKEN_KEY = "ht-token";

    public AdminLoginInterceptor(SysUserService sysUserService, RedisUtil redisUtil) {
        this.sysUserService = sysUserService;
        this.redisUtil = redisUtil;
    }


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        UrlPathHelper urlPathHelper = new UrlPathHelper();
        String urlPath = urlPathHelper.getLookupPathForRequest(request);
        System.out.println(urlPath);
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        if ("/admin/login".equals(urlPath)) {
            return true;
        }


        //下面都是没标注解的情况

        //从header中获取token
        String token = request.getHeader(LOGIN_TOKEN_KEY);
        //如果header中不存在token，则从参数中获取token
        if (StringUtils.isBlank(token)) {
            token = request.getParameter(LOGIN_TOKEN_KEY);
        }
        if (StringUtils.isBlank(token)) {
            token = getToken(request);
        }


        //token为空
        if (StringUtils.isBlank(token)) {
            throw new ApiRRException("请先登录", 401);
        }
        if (request.getAttribute(LOGIN_USER_KEY) == null) {
            //有token
            String userId = redisUtil.get(token);
            if (userId == null) {
                throw new ApiRRException("token:" + token + "登录已过期,请重新登录", 401);
            }
            SysUser user = sysUserService.getById(userId);
            if (user == null) {
                throw new ApiRRException("用户不存在", 401);
            }
            request.setAttribute(LOGIN_USER_KEY, user);
        }
        return true;
    }


    public String getToken(HttpServletRequest request) {
        Cookie[] cookies = request.getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                String name = cookie.getName();
                if (name.equals(LOGIN_TOKEN_KEY)) {
                    return cookie.getValue();
                }
            }
        }

        return null;
    }
}
